package com.docus.server.message.busservice.impl;

import com.docus.core.util.Func;
import com.docus.infrastructure.core.exception.BaseException;
import com.docus.infrastructure.web.api.CommonResult;
import com.docus.infrastructure.web.api.ResultCode;
import com.docus.server.message.busservice.SdBusinessService;
import com.docus.server.message.config.CollectTaskConfig;
import com.docus.server.message.converters.WsNurseSubmitMessageConvert;
import com.docus.server.message.converters.WsRecordSubmitMessageConvert;
import com.docus.server.message.dto.Message;
import com.docus.server.message.dto.MessageResponse;
import com.docus.server.message.dto.RecordSubmitDTO;
import com.docus.server.message.dto.WsNurseSubmitDTO;
import com.docus.server.message.feign.dto.CompensateTasRequest;
import com.docus.server.message.feign.service.CollectTaskService;
import com.docus.server.message.mapper.SdRyIcuMapper;
import com.docus.server.message.mapper.TBasicMapper;
import com.docus.server.message.rpc.ShunDePeopleRpc;
import com.docus.server.message.rpc.dto.HisZyCyYzDTO;
import com.docus.server.message.rpc.request.HisZyCyYzRequest;
import com.docus.server.message.rpc.response.HisZyCyYzResponse;
import com.docus.server.message.validate.RecordSubmitValidate;
import com.docus.server.message.validate.WsNurseSubmitValidate;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.List;
import java.util.Objects;

@Slf4j
@Service
public class SdBusinessServiceImpl implements SdBusinessService {

    private CollectTaskService collectTaskService;
    private TBasicMapper tBasicMapper;
    private ShunDePeopleRpc shunDePeopleRpc;
    private SdRyIcuMapper sdRyIcuMapper;

    @Autowired
    public void setCollectTaskService(CollectTaskService collectTaskService) {
        this.collectTaskService = collectTaskService;
    }

    @Autowired
    public void settBasicMapper(TBasicMapper tBasicMapper) {
        this.tBasicMapper = tBasicMapper;
    }

    @Autowired
    public void setShunDePeopleRpc(ShunDePeopleRpc shunDePeopleRpc) {
        this.shunDePeopleRpc = shunDePeopleRpc;
    }

    @Autowired
    public void setSdRyIcuMapper(SdRyIcuMapper sdRyIcuMapper) {
        this.sdRyIcuMapper = sdRyIcuMapper;
    }

    @Override
    public MessageResponse recordSubmitHandle(Message message) {
        try {
            CollectTaskConfig.TaskConfig taskConfig = CollectTaskConfig.getTaskConfig("WS_RECORD_SUBMIT");
            if (taskConfig == null || Func.isEmpty(taskConfig.getCollectorIds())) {
                throw new RuntimeException("任务配置未配置采集器id,WS_RECORD_SUBMIT");
            }
            RecordSubmitDTO recordSubmitDto = WsRecordSubmitMessageConvert.convert(message.getMessage());
            RecordSubmitValidate.verifyRecordSubmitDTO(recordSubmitDto);
            List<String> patientIds = tBasicMapper.getPatientIdsByInpatientNoAndTimes(recordSubmitDto.getInHospIndexNo(), recordSubmitDto.getVisitNo());
            verifyPatientIds(patientIds);

            if(!isDischargeAndUpdDisDate(patientIds.get(0),recordSubmitDto.getInHospIndexNo(), recordSubmitDto.getVisitNo())){
                throw new BaseException("此患者未出院！");
            }
            tBasicMapper.insertOrUpdateDoctorSubmitTime(patientIds.get(0), recordSubmitDto.getSubmitTime());
            log.info("补偿电子病历任务 patientIds:{} collectId:{}", patientIds, taskConfig.getCollectorIds());
            compensateTask(patientIds, taskConfig.getCollectorIds());
            tBasicMapper.updateEmrSubmitState(patientIds.get(0),1);
            return new MessageResponse(ResultCode.SUCCESS.getCode(), success());
        } catch (BaseException baseException) {
            log.error(baseException.getMessage(), baseException);
            // 业务异常，不希望重试，直接消息返回错误
            MessageResponse response = new MessageResponse(ResultCode.FAILED.getCode(), fail(baseException.getMessage()));
            response.setRetry(0);
            return response;
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
            // 不可知的错误，希望重试
            MessageResponse response = new MessageResponse(ResultCode.FAILED.getCode(), fail("系统错误"));
            response.setRetry(1);
            return response;
        }
    }


    @Override
    public MessageResponse wsNurseSubmitHandle(Message message) {
        try {
            CollectTaskConfig.TaskConfig taskConfig = CollectTaskConfig.getTaskConfig("WS_NURS_SUBMIT");
            if (taskConfig == null || Func.isEmpty(taskConfig.getCollectorIds())) {
                throw new RuntimeException("任务配置未配置采集器id,WS_NURS_SUBMIT");
            }
            WsNurseSubmitDTO nurseSubmitDTO = WsNurseSubmitMessageConvert.convert(message.getMessage());
            WsNurseSubmitValidate.verifyNurseSubmitDTO(nurseSubmitDTO);
            String inHospIndexNo = nurseSubmitDTO.getInHospIndexNo();
            Integer visitNo = nurseSubmitDTO.getVisitNo();
            List<String> patientIds = tBasicMapper.getPatientIdsByInpatientNoAndTimes(inHospIndexNo, visitNo);
            verifyPatientIds(patientIds);

            if(!isDischargeAndUpdDisDate(patientIds.get(0), inHospIndexNo, visitNo)){
                throw new BaseException("此患者未出院！");
            }
            // 如果是重症患者，如果没有重症报告的话，不能提交
            if (isIcuRecord(inHospIndexNo, visitNo)) {
                // 更新重症标识
                confirmAndUpdIcuRecordState(inHospIndexNo,visitNo);
                if (!hasIcuFile(patientIds.get(0))) {
                    throw new BaseException("重症患者，未采集重症文件，不允许提交！");
                }
            }

            tBasicMapper.insertOrUpdateNurseSubmitTime(patientIds.get(0), nurseSubmitDTO.getSubmitTime());
            tBasicMapper.insertOrUpdateNurseFileCount(patientIds.get(0), nurseSubmitDTO.getFilesCount());
            tBasicMapper.insertOrUpdateNurse(patientIds.get(0), nurseSubmitDTO.getQcNurse(),nurseSubmitDTO.getDutyNurse());
            log.info("补偿护理任务 patientIds:{} collectId:{}", patientIds, taskConfig.getCollectorIds());
            compensateTask(patientIds, taskConfig.getCollectorIds());
            return new MessageResponse(ResultCode.SUCCESS.getCode(), success());
        } catch (BaseException baseException) {
            log.error(baseException.getMessage(), baseException);
            // 业务异常，不希望重试，直接消息返回错误
            MessageResponse response = new MessageResponse(ResultCode.FAILED.getCode(), fail(baseException.getMessage()));
            response.setRetry(0);
            return response;
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
            // 不可知的错误，希望重试
            MessageResponse response = new MessageResponse(ResultCode.FAILED.getCode(), fail("系统错误"));
            response.setRetry(1);
            return response;
        }
    }

    /**
     * 根据病案，查找是否有icu来源，未作废的文件数量>0
     * @param patientId 病案主键
     * @return 是否有icu来源，未作废的文件数量>0
     */
    private boolean hasIcuFile(String patientId) {
        String source = "icu";
        return getFileCountByPatientAndSource(patientId, source) > 0;
    }

    /**
     * 根据病案主键和来源，统计未作废的文件数量
     * @param patientId 病案主键
     * @param source 系统/采集来源
     * @return 未作废文件数量
     */
    private int getFileCountByPatientAndSource(String patientId, String source) {
        return tBasicMapper.getFileCountByPatientAndSource(patientId, source);
    }


    @Override
    public void confirmAndUpdIcuRecordState(String inpatientNo, Integer admissTimes) {
        int state = isIcuRecord(inpatientNo, admissTimes) ? 1 : 0;
        tBasicMapper.updIcuRecordState(inpatientNo, admissTimes, state);
    }

    @Override
    public boolean isIcuRecord(String inpatientNo, Integer admissTimes) {
        Integer count = sdRyIcuMapper.countByInpNoAndAdmissTimes(inpatientNo, admissTimes);
        return count>0;
    }

    private void compensateTask(List<String> patientIds, List<String> collectorIds) {
        CompensateTasRequest dto = new CompensateTasRequest();
        dto.setPatientIds(patientIds);
        dto.setCollectorIds(collectorIds);
        dto.setPriority(4);
        CommonResult<String> task = collectTaskService.compensateTask(dto);
        if (ResultCode.FAILED.getCode().equals(task.getCode())) {
            String data = task.getData();
            if (Func.isNotBlank(data) && data.contains("出院")) {
                throw new BaseException("该患者为在院状态，请在患者出院后提交！");
            }
            throw new RuntimeException("补偿失败！");
        }
    }

    private void verifyPatientIds(List<String> patientIds) throws BaseException {
        if (Func.isEmpty(patientIds)) {
            throw new BaseException("住院号+住院次数 ，系统无此患者！");
        }
        if (patientIds.size() > 1) {
            throw new BaseException("住院号+住院次数 ，系统中患者有多个！");
        }
    }

    /**
     * 根据病案主键，获得该患者是否出院,如果没出院，是否更新出院时间再判断是否出院
     *
     * @param patientId 病案主键
     * @return 是否出院
     */
    private boolean isDischargeAndUpdDisDate(String patientId, String inpatientNo, Integer admissTimes) {
        // 第一次，出院则直接判断出院
        int count = tBasicMapper.getDischargeCount(patientId);
        if (count > 0) {
            return true;
        }
        // 没有出院，根据住院出院医嘱，更新出院时间再判断
        HisZyCyYzRequest request = new HisZyCyYzRequest();
        request.setVisitNo(inpatientNo);
        request.setVisitTimes(String.valueOf(admissTimes));
        HisZyCyYzResponse response = shunDePeopleRpc.hisZyCyYz(request);
        // 医嘱没查询到，也表示未出院
        if (isNullYzData(response)) {
            return false;
        }
        HisZyCyYzDTO hisZyCyYzDTO = null;

        List<HisZyCyYzDTO> zyCyYzDtos = response.getData();
        // 3 5 状态可更新时间
        List<String> status = Arrays.asList("3", "5");
        for (HisZyCyYzDTO zyCyYzDTO : zyCyYzDtos) {
            if (status.contains(zyCyYzDTO.getOrder_status())) {
                hisZyCyYzDTO = zyCyYzDTO;
                break;
            }
        }
        // 根据3 5 状态未找到医嘱记录，不更新时间，判定为未出院
        if (Objects.isNull(hisZyCyYzDTO)) {
            return false;
        }

        // 更新出院时间
        tBasicMapper.updateDisDateTime(patientId, hisZyCyYzDTO.getStop_time());
        // 再次判断
        int count2 = tBasicMapper.getDischargeCount(patientId);
        return count2 > 0;
    }

    /**
     * 判断医嘱返回结果是否是空
     * @param response 医嘱返回结果
     * @return 没有医嘱数据放回true
     */
    private boolean isNullYzData(HisZyCyYzResponse response) {
       return Objects.isNull(response) || Func.isEmpty(response.getData()) || Objects.isNull(response.getData().get(0));
    }


    private String success() {
        return "<Response>" +
                "<RetInfo>" +
                "<RetCode>0</RetCode>" +
                "<RetCon>成功</RetCon>" +
                "</RetInfo>" +
                "</Response>";
    }

    private String fail(String message) {
        return "<Response>" +
                "<RetInfo>" +
                "<RetCode>1</RetCode>" +
                "<RetCon>" + message + "</RetCon>" +
                "</RetInfo>" +
                "</Response>";
    }
}
